@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+#
 # Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -178,6 +180,25 @@ class TestGridfs(unittest.TestCase):
         self.fs.delete(a)
         self.assertRaises(NoFile, self.fs.get_last_version, "test")
 
+    def test_get_version(self):
+        self.fs.put("foo", filename="test")
+        time.sleep(0.01)
+        self.fs.put("bar", filename="test")
+        time.sleep(0.01)
+        self.fs.put("baz", filename="test")
+        time.sleep(0.01)
+
+        self.assertEqual("foo", self.fs.get_version("test", 0).read())
+        self.assertEqual("bar", self.fs.get_version("test", 1).read())
+        self.assertEqual("baz", self.fs.get_version("test", 2).read())
+
+        self.assertEqual("baz", self.fs.get_version("test", -1).read())
+        self.assertEqual("bar", self.fs.get_version("test", -2).read())
+        self.assertEqual("foo", self.fs.get_version("test", -3).read())
+
+        self.assertRaises(NoFile, self.fs.get_version, "test", 3)
+        self.assertRaises(NoFile, self.fs.get_version, "test", -4)
+
     def test_put_filelike(self):
         oid = self.fs.put(StringIO("hello world"), chunk_size=1)
         self.assertEqual(11, self.db.fs.chunks.count())
@@ -212,6 +233,17 @@ class TestGridfs(unittest.TestCase):
         self.assertFalse(self.fs.exists(foo={"$gt": 12}))
         self.assertFalse(self.fs.exists({"foo": {"$gt": 12}}))
 
+    def test_put_unicode(self):
+        self.assertRaises(TypeError, self.fs.put, u"hello")
+
+        oid = self.fs.put(u"hello", encoding="utf-8")
+        self.assertEqual("hello", self.fs.get(oid).read())
+        self.assertEqual("utf-8", self.fs.get(oid).encoding)
+
+        oid = self.fs.put(u"aé", encoding="iso-8859-1")
+        self.assertEqual(u"aé".encode("iso-8859-1"), self.fs.get(oid).read())
+        self.assertEqual("iso-8859-1", self.fs.get(oid).encoding)
+
 
 if __name__ == "__main__":
     unittest.main()
