@@ -39,17 +39,15 @@ class TestMasterSlaveConnection(unittest.TestCase):
         self.slaves = []
         try:
             self.slaves.append(Connection(os.environ.get("DB_IP2", host),
-                                          int(os.environ.get("DB_PORT2",
-                                                             27018)),
-                                          slave_okay=True))
+                               int(os.environ.get("DB_PORT2", 27018)),
+                               slave_okay=True))
         except ConnectionFailure:
             pass
 
         try:
             self.slaves.append(Connection(os.environ.get("DB_IP3", host),
-                                          int(os.environ.get("DB_PORT3",
-                                                             27019)),
-                                          slave_okay=True))
+                               int(os.environ.get("DB_PORT3", 27019)),
+                               slave_okay=True))
         except ConnectionFailure:
             pass
 
